import Singleton from '../Base/Singleton'
import { ApiMsgEnum } from '../Common'
import PlayerManager from './PlayerManager'
import Room from './Room'

export default class RoomManager extends Singleton {
  static get Instance() {
    return super.GetInstance<RoomManager>()
  }

  roomId = 1
  rooms: Set<Room> = new Set()
  idMapRoom: Map<number, Room> = new Map()

  createRoom() {
    const room = new Room(this.roomId++)
    this.rooms.add(room)
    this.idMapRoom.set(room.id, room)
    return room
  }

  joinRoom(rid: number, uid: number) {
    const room = this.getRoomById(rid)
    if (room) {
      room.join(uid)
      return room
    }
  }

  leaveRoom(rid: number, uid: number) {
    const room = this.getRoomById(rid)
    if (room) {
      room.leave(uid)
    }
  }

  closeRoom(rid: number) {
    const room = this.getRoomById(rid)
    if (room) {
      this.rooms.delete(room)
      this.idMapRoom.delete(rid)
    }
  }

  startRoom(rid: number) {
    const room = this.getRoomById(rid)
    if (room) {
      room.start()
    }
  }

  getRoomById(id: number) {
    return this.idMapRoom.get(id)
  }

  syncRooms() {
    for (const player of PlayerManager.Instance.players) {
      player.connection.sendMsg(ApiMsgEnum.MsgRoomList, { list: this.getRoomsView() })
    }
  }

  syncRoom(rid: number) {
    const room = this.idMapRoom.get(rid)
    if (room) {
      room.sync()
    }
  }

  getRoomsView(rooms: Set<Room> = this.rooms) {
    return [...rooms].map((room) => this.getRoomView(room))
  }

  getRoomView({ id, players }: Room) {
    return { id, players: PlayerManager.Instance.getPlayersView(players) }
  }
}
