import WebSocket from 'ws';
import { EventEmitter } from 'stream';
import { MyServer } from './MyServer';
import { getTime, toArrayBuffer } from '../Utils';
import { ApiMsgEnum, IModel } from '../Common';
import { binaryEncode, binaryDecode } from '../Common/Binary';

export enum ConnectionEventEnum {
  Close = 'Close',
}

export class Connection extends EventEmitter {
  server: MyServer
  ws: WebSocket
  msgMap: Map<ApiMsgEnum, Function[]> = new Map()
  playerId?: number;

  constructor(server: MyServer, ws: WebSocket) {
    super()

    this.server = server
    this.ws = ws
    this.ws.on('close', (code: number, reason: Buffer) => {
      this.emit(ConnectionEventEnum.Close, code, reason.toString())
    })

    this.ws.on('message', (buffer: Buffer) => {
      // const str = buffer.toString()
      try {
        const json = binaryDecode(toArrayBuffer(buffer))
        const { name, data } = json
        // console.log(`${getTime()}接收|字节数${buffer.length}|${this.playerId || -1}|${str}`)
        console.log(`${getTime()}接收|字节数${buffer.length}|${this.playerId || -1}|${JSON.stringify(json)}`)
        if (this.server.apiMap.has(name)) {
          try {
            const cb = this.server.apiMap.get(name)
            const res = cb?.(this, data)
            this.sendMsg(name, {
              success: true,
              res,
            })
          } catch (error) {
            this.sendMsg(name, {
              success: false,
              error: (error as Error)?.message,
            })
          }
          return
        }

        if (this.msgMap.has(name)) {
          this.msgMap.get(name)?.forEach(cb => cb(data))
        }
      } catch (error) {
        // console.log(`解析失败，${str}不是合法的JSON格式：`, error)
        console.log(error)
      }
    })
  }

  listenMsg<T extends keyof IModel['msg']>(name: T, cb: (args: IModel['msg'][T]) => void) {
    if (this.msgMap.has(name)) {
      this.msgMap.get(name)?.push(cb)
    } else {
      this.msgMap.set(name, [cb])
    }
  }

  unlistenMsg<T extends keyof IModel['msg']>(name: T, cb: (args: IModel['msg'][T]) => void) {
    if (this.msgMap.has(name)) {
      const index = this.msgMap.get(name)?.indexOf(cb) || -1
      index > -1 && this.msgMap.get(name)?.splice(index, 1)
    }
  }

  sendMsg<T extends keyof IModel['msg']>(name: T, data: IModel['msg'][T]) {
    const msg = JSON.stringify({
      name,
      data
    })
    const view = binaryEncode(name, data)
    const buffer = Buffer.from(view.buffer)
    console.log(`${getTime()}发送|字节数${buffer.length}|${this.playerId || -1}|${msg}`)
    this.ws.send(buffer)
  }
}